﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Controls;

namespace WpfApp17
{
    public partial class MainWindow : Window
    {
        private readonly List<Color> colorsCollection = GetColors();
        private MainViewModel model;

        public MainWindow()
        {
            InitializeComponent();
            this.model = new MainViewModel();

            this.SetAxesRange(0, 1200, 0, 250);
            this.highResSeries.ItemsSource = this.model.HighResPoints;
            this.highResColorizer.GradientStops = GetGradientStops(0, 63);
        }

        private void SetAxesRange(double minX, double maxX, double minY, double maxY)
        {
            this.xAxis.Minimum = minX;
            this.xAxis.Maximum = maxX;
            this.yAxis.Minimum = minY;
            this.yAxis.Maximum = maxY;
        }

        private void RadSlider_DragStarted(object sender, RadDragStartedEventArgs e)
        {
            this.highResSeries.ItemsSource = null;
            this.lowResSeries.ItemsSource = this.model.LowResPoints;
        }

        private void RadSlider_DragCompleted(object sender, RadDragCompletedEventArgs e)
        {         
            this.highResSeries.ItemsSource = this.model.HighResPoints;
            this.lowResSeries.ItemsSource = null;
            this.highResColorizer.GradientStops = GetGradientStops((int)e.SelectionStart, (int)e.SelectionEnd);
        }

        private void RadSlider_DragDelta(object sender, RadDragDeltaEventArgs e)
        {
            this.lowResColorizer.GradientStops = GetGradientStops((int)e.SelectionStart, (int)e.SelectionEnd);
        }

        private GradientStopCollection GetGradientStops(int startIndex, int endIndex)
        {
            var colors = colorsCollection.Skip(startIndex).Take(endIndex - startIndex);
            var step = 1.0 / colors.Count();
            var stops = colors.Select((color, index) => new GradientStop(color, index * step));
            return new GradientStopCollection(stops);
        }

        private static List<Color> GetColors()
        {
            var result = new List<Color>();
            int rMax = Colors.Red.R;
            int rMin = Colors.Blue.R;
            int bMax = Colors.Red.B;
            int bMin = Colors.Blue.B;
            int gMax = Colors.Red.G;
            int gMin = Colors.Blue.G;
            int size = 64;

            for (int i = 0; i < size; i++)
            {
                var rAverage = rMin + (int)((rMax - rMin) * i / size);
                var gAverage = gMin + (int)((gMax - gMin) * i / size);
                var bAverage = bMin + (int)((bMax - bMin) * i / size);
                result.Add(Color.FromRgb((byte)rAverage, (byte)gAverage, (byte)bAverage));
            }

            return result;
        }       
    }

    public class MainViewModel : ViewModelBase
    {
        public ObservableCollection<PlotInfo> HighResPoints { get; set; }
        public ObservableCollection<PlotInfo> LowResPoints { get; set; }        


        public MainViewModel()
        {
            this.HighResPoints = GetData(1000, 200, 1);
            this.LowResPoints = GetData(1000, 200, 3);
        }

        private static ObservableCollection<PlotInfo> GetData(int maxX, int maxY, int step)
        {
            var result = new ObservableCollection<PlotInfo>();           
            for (int x = 0; x < maxX; x+=step)
            {
                for (int y = 0; y < maxY; y+=step)
                {
                    double xValue = Math.Sin(x * Math.PI / (0.25 * maxX));
                    double yValue = Math.Cos(y * Math.PI / (0.50 * maxY));
                    double z = 200 * xValue * yValue;
                    PlotInfo pi = new PlotInfo { X = x, Y = y, Z = z, };
                    result.Add(pi);
                }
            }
            return result;
        }
    }

    public class PlotInfo
    {
        public double X { get; set; }
        public double Y { get; set; }
        public double Z { get; set; }
    }
}
